import json
from terminal import *
from .util import *


IND = 'No.'

def print_title(s):
    print('\n%s' % bold(magenta(s)))


def print_arr(arr, cols=[], show_no=True):
    '''
    arr: result rows
    cols: shown fields. example: ['ImageId', 'ImageName', ...]
    '''
    if len(arr)==0:
        print('Empty array')
        return False

    if len(cols)==0:
        return __print_all_fields(arr)
    else:
        return __print_col_fields(arr, cols, show_no)



def print_task_desc(task_name, task_desc):
    t = []

    t.append('%s' % gray('-'*90))

    t.append('\n%s: %s' % (gray('TaskName'), task_name) )
    t.append('\t%s: %s' % (gray('ImageId'), task_desc['ImageId']) )
    t.append('\t%s: %s' % (gray('InstanceCount') , task_desc['InstanceCount']) )

    t.append('\n%s: %s' % (gray('ProgramType') ,task_desc['ProgramType']))
    t.append('\t%s: %s' % (gray('ProgramName') ,task_desc['ProgramName']))
    t.append('\t%s: %s' % (gray('ProgramArguments') ,task_desc['ProgramArguments'] ))

    t.append('\n%s: %s' % (gray('PackageUri') ,task_desc['PackageUri'] ))
    t.append('\n%s: %s' % (gray('StdoutRedirectPath') ,task_desc['StdoutRedirectPath'] ))
    t.append('\n%s: %s' % (gray('StderrRedirectPath') ,task_desc['StderrRedirectPath'] ))

    t.append('\n%s: %s' % (gray('Timeout') ,task_desc['Timeout'] ))
    t.append('\t%s: %s' % (gray('CPU'), task_desc['ResourceDescription']['Cpu'] ))
    t.append('\t%s: %s' % (gray('Memory'), task_desc['ResourceDescription']['Memory'] ))

    t.append('\n%s:' % (gray('EnvironmentVariables')) )
    for k in task_desc['EnvironmentVariables']:
        t.append('  %s : %s' % (k, task_desc['EnvironmentVariables'][k]))

    t.append('\n%s: %s' % (gray('OssMappingLock'), task_desc['OssMappingLock'] ))
    t.append('\t%s: %s' % (gray('OssMappingLocale'), task_desc['OssMappingLocale'] ))

    t.append('\n%s:' % (gray('OssMapping')) )
    for k in task_desc['OssMapping']:
        t.append('  %s : %s' % (k, task_desc['OssMapping'][k]))
    t.append('\n')
    t.append('%s' % gray('-'*90))

    print(''.join(t))

def print_instance_blocks(arr):
    t=[]
    c=0
    for item in arr:
        c+=1;
        ins = to_dict(item, INSTANCE_COLS)

        id = str(ins['InstanceId'])
        color_state = get_color_state(ins['State'])
        t.append('%s:%s' % (id, color_state) )
        t.append('\n' if (c%7==0) else '\t')

    print(''.join(t))

def __print_col_fields(arr, cols, show_no=True):
    """
    only print the expected columns in each item
    """
    # calc
    col_max_len_map = {}
    for c in cols:
        col_max_len_map[c] = len(c)

    for item in arr:
        for c in cols:
            v = item.get('ProgressTxt') if c=='Progress' else item.get(c)
            col_max_len_map[c] = max( len(__to_str(v)),  col_max_len_map[c] )

    if show_no:
        col_max_len_map[IND] = max( len(IND), len(str(len(arr)) ) )

    # print cols
    t = []
    t2 = []
    t3 = []

    if show_no:
        t2.append('%s%s' % (IND, __fit_len(IND, col_max_len_map[IND])))
        t3.append('-' * col_max_len_map[IND] )

    for c in cols:
        t2.append( '%s%s' % (c,  __fit_len(c, col_max_len_map[c])))
        t3.append('-'*col_max_len_map[c])

    t.append('| %s |' % ' | '.join(t2))
    t.append('| %s |' % ' | '.join(t3))


    # print data
    for (i,item) in enumerate(arr):
        t2 = []

        if show_no:
            t2.append('%s%s' % (i+1 , __fit_len(i+1, col_max_len_map[IND])));

        for c in cols:
            v = __to_str(item.get(c) or '')

            if c=='State':
                v2 = get_color_state(v)
                t2.append( '%s%s' % (v2 , __fit_len(v, col_max_len_map[c])))

            elif c=='Progress':
                t2.append('%s%s' % (item.get('Progress'),  __fit_len(item.get('ProgressTxt'), col_max_len_map[c])))

            else:
                t2.append( '%s%s' % (v , __fit_len(v, col_max_len_map.get(c))) )

        t.append('| %s |' % ' | '.join(t2))

    print('\n'.join(t))
    return len(t)


def __print_all_fields(arr):
    """
    print all fields in each item
    """
    cols = []
    for k in arr[0].keys():
        cols.append(k)

    t = []
    for (i,item) in enumerate(arr):
        t2=[]
        for c in cols:
            v = json.dumps(item.get(c)) if type(item.get(c)) != str else item.get(c)
            t2.append(' %s: %s' % (c, (get_color_state(v) if c=='State' else v)) )
        t.append('%s\n%d.---\n%s' % ( '=' * 20 , i+1, '\n'.join(t2) ) )
    print('\n'.join(t))
    return len(t)


def done(s=None):
    print(green('%s' % (s or 'Done')))

def total(c):
    done('Total %d rows.' % c)

def print_count(c):
    done('Found %d rows.' % c)

def tip(s):
    print(gray('\n  %s\n' % s))

def __to_str(s):
    return s if type(s)==str or type(s)==unicode else json.dumps(s)

def __fit_len(s, spacerLen):
    n = spacerLen - len(__to_str(s))
    return ' ' * n
